import java.util.Arrays;
import java.util.LinkedList;
import java.util.Map;

/**
 * Mark&sweep garbage collector implementation
 *
 * @author Kuba Madej
 *
 */
public class CollectorImpl implements Collector {
	/**
	 * Variables map
	 */
	private Map<String, Integer> variables;

	/**
	 * Constructs new instance
	 *
	 * @param variables
	 *            variables map
	 */
	public CollectorImpl(Map<String, Integer> variables) {
		this.variables = variables;
	}

	/**
	 *
	 * @see Collector#collect(int[])
	 */
	public void collect(int[] heap) {
		final int maxIndex = heap.length - 4;
		for (int objectPointer : variables.values()) {
			if (objectPointer != 0)
				mark(objectPointer, heap);
		}

		for (int objectIndex = HeapConstants.BASE_INDEX; objectIndex <= maxIndex; objectIndex += 4)
			if (heap[objectIndex] == ObjectHeader.MARKED)
				heap[objectIndex] = ObjectHeader.UNMARKED;
			else
				Arrays.fill(heap, objectIndex, objectIndex + 4, 0);
	}

	/**
	 * Iterativly marks objects as accessible starting from the object given
	 *
	 * @param objectIndex
	 *            index of the object to start from
	 * @param heap
	 *            heap to work on
	 */
	private void mark(int objectIndex, int[] heap) {
		final LinkedList<Integer> stack = new LinkedList<Integer>();

		stack.add(objectIndex);

		while (!stack.isEmpty()) {
			objectIndex = stack.poll();
			if (heap[objectIndex] != ObjectHeader.MARKED) {
				heap[objectIndex] = ObjectHeader.MARKED;
				int pointer = heap[objectIndex + HeapConstants.F1];
				if (pointer != 0)
					stack.add(pointer);
				pointer = heap[objectIndex + HeapConstants.F2];
				if (pointer != 0)
					stack.add(pointer);
			}
		}
	}
}
