import java.util.Map;

/**
 * Variable declaration statement implementation
 *
 * @author Kuba Madej
 *
 */
public class VariableDeclarationStatement implements Statement {
	/**
	 * Heap to work on
	 */
	private int[] heap;

	/**
	 * Variables map
	 */
	private Map<String, Integer> variables;

	/**
	 * Name of a variable to declare
	 */
	private String name;

	/**
	 * Garbage collector instance
	 */
	private Collector collector;

	/**
	 * Constructs new instance
	 *
	 * @param heap
	 *            heap to work on
	 * @param variables
	 *            variables map
	 * @param collector
	 *            garbage collector to use
	 * @param name
	 *            name of the variable to declare
	 */
	public VariableDeclarationStatement(int[] heap,
			Map<String, Integer> variables, final Collector collector,
			final String name) {
		this.heap = heap;
		this.variables = variables;
		this.name = name;
		this.collector = collector;
	}

	/**
	 *
	 * @see Statement#execute()
	 */
	public void execute() {
		int i = allocateMemory();
		if (i == 0) {
			NPJ.collect(heap, collector);
			i = allocateMemory();
			if (i == 0)
				throw new RuntimeException("Out of heap space");
		}
		heap[i] = ObjectHeader.UNMARKED;
		variables.put(name, i);
	}

	/**
	 * Locates a mamory cell where an object can be placed
	 *
	 * @return 1-based heap index or 0 if out of memory
	 */
	private int allocateMemory() {
		final int maxIndex = heap.length - 4;
		int i;
		for (i = HeapConstants.BASE_INDEX; i <= maxIndex
				&& heap[i] != ObjectHeader.UNUSED; i += 4)
			;
		return i > maxIndex ? 0 : i;
	}
}
