import java.io.File;
import java.io.FileNotFoundException;
import java.util.Scanner;

/**
 * Main class of the whole interpreter
 *
 * @author Kuba Madej
 *
 */
public class Interpreter {
	/**
	 * Process code returned in case of failure
	 */
	private static final int FAILURE_CODE = 1;

	/**
	 * Heap size property name
	 */
	private static final String HEAP_SIZE_PROPERTY_NAME = "npj.heap.size";

	/**
	 * Constructs new instance and parses program contained in the file given
	 *
	 * @param fileName
	 *            file name containing program to interpret
	 */
	public Interpreter(final String fileName) {
		Scanner scanner = null;
		try {
			scanner = new Scanner(new File(fileName));
		} catch (FileNotFoundException e) {
			System.err.println("File " + fileName + " not found");
			System.exit(FAILURE_CODE);
		}
		scanner.useDelimiter(";");

		StatementFactory statementFactory = new StatementFactory(
				new int[Integer.parseInt(System
						.getProperty(HEAP_SIZE_PROPERTY_NAME))]);

		String statement = "";
		while (scanner.hasNext()) {
			statement += scanner.next();
			final int firstIndexOfEscape = statement.indexOf("\"");
			if (firstIndexOfEscape != -1
					&& statement.lastIndexOf("\"") == firstIndexOfEscape) {
				statement += ";";
				continue;
			}
			statementFactory.getStatement(statement.trim()).execute();
			statement = "";
		}
	}

	/**
	 * Runs the interpreter
	 *
	 * @param args
	 *            command line arguments
	 */
	public static void main(String[] args) {
		new Interpreter(args[0]);
	}

}
