import java.util.Map;

/**
 * Variable declaration statement implementation
 *
 * @author Kuba Madej
 *
 */
public class VariableDeclarationStatement implements Statement {
	/**
	 * Heap to work on
	 */
	private int[] heap;

	/**
	 * Variables map
	 */
	private Map<String, Integer> variables;

	/**
	 * Name of a variable to declare
	 */
	private String name;

	/**
	 * Garbage collector instance
	 */
	private CollectorImpl collector;

	/**
	 * Constructs new instance
	 *
	 * @param heap
	 *            heap to work on
	 * @param variables
	 *            variables map
	 * @param collector
	 *            garbage collector to use
	 * @param name
	 *            name of the variable to declare
	 */
	public VariableDeclarationStatement(int[] heap,
			Map<String, Integer> variables, final CollectorImpl collector,
			final String name) {
		this.heap = heap;
		this.variables = variables;
		this.name = name;
		this.collector = collector;
	}

	/**
	 *
	 * @see Statement#execute()
	 */
	public void execute() {
		int i = collector.allocateMemory(heap);
		if (i == 0) {
			NPJ.collect(heap, collector);
			i = collector.allocateMemory(heap);
			if (i == 0)
				throw new RuntimeException("Out of heap space");
		}
		variables.put(name, i);
	}
}
